@@ -24,10 +24,11 @@
 #ifndef CONE_H_
 #define CONE_H_
 
+#include <sys/stat.h>
 #include <vector>
 #include <map>
 #include <utility> //for pair
-//#include <boost/dynamic_bitset.hpp>
+#include <boost/dynamic_bitset.hpp>
 
 #include <libnormaliz/libnormaliz.h>
 #include <libnormaliz/cone_property.h>
@@ -82,6 +83,8 @@ public:
 //---------------------------------------------------------------------------
 //                    Constructors, they preprocess the input
 //---------------------------------------------------------------------------
+    
+    Cone(); //default constructor
 
     /* give up to 3 matrices as input
      * the types must be pairwise different
@@ -112,6 +115,21 @@ public:
 
     /* give multiple input */
     Cone(const map< InputType , vector< vector<mpq_class> > >& multi_input_data);
+    
+//-----------------------------------------------------------------------------
+// the same for nmz_float
+    
+    Cone(InputType type, const vector< vector<nmz_float> >& input_data);
+
+    Cone(InputType type1, const vector< vector<nmz_float> >& input_data1,
+         InputType type2, const vector< vector<nmz_float> >& input_data2);
+
+    Cone(InputType type1, const vector< vector<nmz_float> >& input_data1,
+         InputType type2, const vector< vector<nmz_float> >& input_data2,
+         InputType type3, const vector< vector<nmz_float> >& input_data3);
+
+    /* give multiple input */
+    Cone(const map< InputType , vector< vector<nmz_float> > >& multi_input_data);
 
 //-----------------------------------------------------------------------------
 // Now with Matrix
@@ -143,6 +161,21 @@ public:
     /* give multiple input */
     Cone(const map< InputType , Matrix<mpq_class> >& multi_input_data);
     
+//-----------------------------------------------------------------------------
+// Now with Matrix and nmz_float
+    
+    Cone(InputType type, const Matrix<nmz_float>& input_data);
+
+    Cone(InputType type1, const Matrix<nmz_float>& input_data1,
+         InputType type2, const Matrix<nmz_float>& input_data2);
+
+    Cone(InputType type1, const Matrix<nmz_float>& input_data1,
+         InputType type2, const Matrix<nmz_float>& input_data2,
+         InputType type3, const Matrix<nmz_float>& input_data3);
+
+    /* give multiple input */
+    Cone(const map< InputType , Matrix<nmz_float> >& multi_input_data);
+    
     
 //---------------------------------------------------------------------------
 //                                Destructor
@@ -167,9 +200,7 @@ public:
 //---------------------------------------------------------------------------
 
     // return what was NOT computed
-    // ConeProperties compute(ComputationMode mode = Mode::hilbertBasisSeries); //default: everything
-    ConeProperties compute_inner(ConeProperties ToCompute);
-    // special case for up to 3 CPs
+    // special cases for up to 3 CPs
     ConeProperties compute(ConeProperties ToCompute);
     ConeProperties compute(ConeProperty::Enum);
     ConeProperties compute(ConeProperty::Enum, ConeProperty::Enum);
@@ -203,6 +234,7 @@ public:
     
     Cone<Integer>& getIntegerHullCone() const;
     Cone<Integer>& getSymmetrizedCone() const;
+    Cone<Integer>& getProjectCone() const;
 
     const Matrix<Integer>& getGeneratorsMatrix();
     const vector< vector<Integer> >& getGenerators();
@@ -211,6 +243,14 @@ public:
     const Matrix<Integer>& getExtremeRaysMatrix();
     const vector< vector<Integer> >& getExtremeRays();
     size_t getNrExtremeRays();
+    
+    const Matrix<nmz_float>& getVerticesFloatMatrix();
+    const vector< vector<nmz_float> >& getVerticesFloat();
+    size_t getNrVerticesFloat();
+    
+    const Matrix<nmz_float>& getSuppHypsFloatMatrix();
+    const vector< vector<nmz_float> >& getSuppHypsFloat();
+    size_t getNrSuppHypsFloat();
 
     const Matrix<Integer>& getVerticesOfPolyhedronMatrix();
     const vector< vector<Integer> >& getVerticesOfPolyhedron();
@@ -235,6 +275,7 @@ public:
     Integer getTriangulationDetSum();
 
     vector<Integer> getWitnessNotIntegrallyClosed();
+    vector<Integer> getGeneratorOfInterior();
 
     const Matrix<Integer>& getHilbertBasisMatrix();
     const vector< vector<Integer> >& getHilbertBasis();
@@ -261,6 +302,8 @@ public:
     vector<Integer> getClassGroup();
 
     mpq_class getMultiplicity();
+    mpq_class getVolume();
+    nmz_float getEuclideanVolume();
     mpq_class getVirtualMultiplicity();
     mpq_class getIntegral();
     const pair<HilbertSeries, mpz_class>& getWeightedEhrhartSeries();
@@ -274,6 +317,7 @@ public:
     bool isDeg1ExtremeRays();
     bool isDeg1HilbertBasis();
     bool isIntegrallyClosed();
+    bool isGorenstein();
     bool isReesPrimary();
     Integer getReesPrimaryMultiplicity();
     const Matrix<Integer>& getOriginalMonoidGeneratorsMatrix();
@@ -299,11 +343,34 @@ public:
     void set_output_dir(string name);
     
     void setPolynomial(string poly);
+    void setNrCoeffQuasiPol(long nr_coeff);
+    void setExpansionDegree(long degree);
     
     bool get_verbose ();
     
     IntegrationData& getIntData();
+
+    void resetGrading(vector<Integer> lf);
+
+    const Matrix<Integer>& getMatrixConePropertyMatrix(ConeProperty::Enum property);
+    const vector< vector<Integer> >& getMatrixConeProperty(ConeProperty::Enum property);
+
+    const Matrix<nmz_float>& getFloatMatrixConePropertyMatrix(ConeProperty::Enum property);
+    const vector< vector<nmz_float> >& getFloatMatrixConeProperty(ConeProperty::Enum property);
     
+    vector<Integer> getVectorConeProperty(ConeProperty::Enum property);
+
+    Integer getIntegerConeProperty(ConeProperty::Enum property);
+
+    mpz_class getGMPIntegerConeProperty(ConeProperty::Enum property);
+    
+    mpq_class getRationalConeProperty(ConeProperty::Enum property);
+
+    nmz_float getFloatConeProperty(ConeProperty::Enum property);
+
+    size_t getMachineIntegerConeProperty(ConeProperty::Enum property);
+
+    bool getBooleanConeProperty(ConeProperty::Enum property);
 
 //---------------------------------------------------------------------------
 //                          private part
@@ -311,13 +378,11 @@ public:
 
 private:
     
-    bool already_in_compute; // protection against call of compute within compute
-                             // such calls must go via recursive_compute.
-    
     string project;
     string output_dir;
     string nmz_call;
     size_t dim;
+    bool inhom_input;
     
     // the following three matrices store the constraints of the input
     Matrix<Integer> Inequalities;
@@ -336,9 +401,11 @@ private:
     Matrix<Integer> OriginalMonoidGenerators;
     Matrix<Integer> Generators;
     Matrix<Integer> ExtremeRays;
+    Matrix<nmz_float> VerticesFloat;
     vector<bool> ExtremeRaysIndicator;
     Matrix<Integer> VerticesOfPolyhedron;
     Matrix<Integer> SupportHyperplanes;
+    Matrix<nmz_float> SuppHypsFloat;
     Matrix<Integer> ExcludedFaces;
     Matrix<Integer> PreComputedSupportHyperplanes;
     size_t TriangulationSize;
@@ -347,14 +414,19 @@ private:
     bool triangulation_is_partial;
     vector< pair<vector<key_t>, Integer> > Triangulation;
     vector<vector<bool> > OpenFacets;
+    vector<bool> projection_coord_indicator;
     vector< pair<vector<key_t>, long> > InExData;
     list< STANLEYDATA_int > StanleyDec;
     list< STANLEYDATA<Integer> > StanleyDec_export;
     mpq_class multiplicity;
+    mpq_class volume;
+    double euclidean_volume;
     mpq_class Integral;
     mpq_class VirtualMultiplicity;
     vector<Integer> WitnessNotIntegrallyClosed;
+    vector<Integer> GeneratorOfInterior;
     Matrix<Integer> HilbertBasis;
+    Matrix<Integer> HilbertBasisRecCone;
     Matrix<Integer> BasisMaxSubspace;
     Matrix<Integer> ModuleGeneratorsOverOriginalMonoid;
     Matrix<Integer> Deg1Elements;
@@ -365,13 +437,21 @@ private:
     Integer GradingDenom;
     Integer index;  // the internal index
     Integer unit_group_index;
+    
+    vector<boost::dynamic_bitset<> > Pair; // for indicator vectors in project-and_lift
+    vector<boost::dynamic_bitset<> > ParaInPair; // if polytope is a parallelotope
+    bool check_parallelotope();
+    bool is_parallelotope;
 
     bool pointed;
     bool inhomogeneous;
+    bool gorensetin;
     bool deg1_extreme_rays;
     bool deg1_hilbert_basis;
     bool integrally_closed;
+    bool Gorenstein;
     bool rees_primary;
+    bool dual_original_generators; // true means: dual cone has original generators
     Integer ReesPrimaryMultiplicity;
     int affine_dim; //dimension of polyhedron
     size_t recession_rank; // rank of recession monoid
@@ -381,11 +461,6 @@ private:
     
     bool is_approximation;
     Cone* ApproximatedCone;
-    
-    // some properties of the current computation taken from ToCompute
-    bool explicit_HilbertSeries; // true = Hilbert series set explicitly and not only via default mode
-    bool naked_dual; // true = dual mode set, but neither Hilbert basis nor deg 1 points
-    bool default_mode; // true default mode set
 
     Matrix<Integer> WeightsGrad;
     vector<bool> GradAbs;
@@ -396,8 +471,13 @@ private:
     // if this is true we allow to change to a smaller integer type in the computation
     bool change_integer_type;
     
-    Cone<Integer>* IntHullCone;
-    Cone<Integer>* SymmCone;
+    Cone<Integer>* IntHullCone; // cone containing data of integer hull
+    Cone<Integer>* SymmCone;    // cone containing symmetrized data
+    Cone<Integer>* ProjCone;    // cone containing projection to selected coordinates 
+    
+    // In cone based algorithms we use the following information
+    bool Grading_Is_Coordinate; // indicates that the grading or dehomogenization is a coordinate
+    key_t GradingCoordinate;  // namely this one
 
     void compose_basis_change(const Sublattice_Representation<Integer>& SR); // composes SR
 
@@ -405,6 +485,7 @@ private:
     void process_multi_input(const map< InputType, vector< vector<Integer> > >& multi_input_data);
     void process_multi_input_inner(map< InputType, vector< vector<Integer> > >& multi_input_data);
     void process_multi_input(const map< InputType, vector< vector<mpq_class> > >& multi_input_data);
+    void process_multi_input(const map< InputType, vector< vector<nmz_float> > >& multi_input_data);
     
     void prepare_input_lattice_ideal(map< InputType, vector< vector<Integer> > >& multi_input_data);
     void prepare_input_constraints(const map< InputType, vector< vector<Integer> > >& multi_input_data);
@@ -422,14 +503,12 @@ private:
     void check_vanishing_of_grading_and_dehom();
     void process_lattice_data(const Matrix<Integer>& LatticeGenerators, Matrix<Integer>& Congruences, Matrix<Integer>& Equations);
     
-    ConeProperties recursive_compute(ConeProperties ToCompute);
-    
     void try_symmetrization(ConeProperties& ToCompute);
-    void try_approximation (ConeProperties& ToCompute);
+    void try_approximation_or_projection (ConeProperties& ToCompute);
 
     Matrix<Integer> prepare_input_type_2(const vector< vector<Integer> >& Input);
     Matrix<Integer> prepare_input_type_3(const vector< vector<Integer> >& Input);
-    void prepare_input_type_4(Matrix<Integer>& Inequalities);
+    void insert_default_inequalities(Matrix<Integer>& Inequalities);
 
     /* only used by the constructors */
     void initialize();
@@ -438,9 +517,9 @@ private:
     void compute_full_cone(ConeProperties& ToCompute);
 
     /* compute the generators using the support hyperplanes */
-    void compute_generators();
+    void compute_generators(ConeProperties& ToCompute);
     template<typename IntegerFC>
-    void compute_generators_inner();
+    void compute_generators_inner(ConeProperties& ToCompute);
 
     /* compute method for the dual_mode, used in compute(mode) */
     void compute_dual(ConeProperties& ToCompute);
@@ -453,10 +532,10 @@ private:
     template<typename IntegerFC>
     void extract_data(Full_Cone<IntegerFC>& FC);
     template<typename IntegerFC>
-    void extract_supphyps(Full_Cone<IntegerFC>& FC);
+    void extract_supphyps(Full_Cone<IntegerFC>& FC, Matrix<Integer>& ret, bool dual=true);
+    void extract_supphyps(Full_Cone<Integer>& FC, Matrix<Integer>& ret, bool dual=true);    
     
-    void extract_supphyps(Full_Cone<Integer>& FC);
-
+    void norm_dehomogenization(size_t FC_dim);
 
     /* set OriginalMonoidGenerators */
     void set_original_monoid_generators(const Matrix<Integer>&);
@@ -470,6 +549,8 @@ private:
     void compute_unit_group_index();
     /* try to find a witness for not integrally closed in the Hilbert basis */
     void find_witness();
+    
+    void check_Gorenstein (ConeProperties&  ToCompute);
 
     Integer compute_primary_multiplicity();
     template<typename IntegerFC>
@@ -479,16 +560,38 @@ private:
     void complete_sublattice_comp(ConeProperties& ToCompute); // completes the sublattice computations
     void complete_HilbertSeries_comp(ConeProperties& ToCompute);
     
+    void treat_polytope_as_being_hom_defined(ConeProperties ToCompute);
+    
     void compute_integral (ConeProperties& ToCompute);
     void compute_virt_mult (ConeProperties& ToCompute);
     void compute_weighted_Ehrhart(ConeProperties& ToCompute);
     
+    void compute_vertices_float(ConeProperties& ToCompute);
+    void compute_supp_hyps_float(ConeProperties& ToCompute);
+    
     void make_StanleyDec_export();
     
     void NotComputable (string message); // throws NotComputableException if default_mode = false
     
+    void set_parallelization();
+    
     template<typename IntegerFC>
     void give_data_of_approximated_cone_to(Full_Cone<IntegerFC>& FC);
+    
+    void project_and_lift(ConeProperties& ToCompute, Matrix<Integer>& Deg1, const Matrix<Integer>& Gens, Matrix<Integer>& Supps, bool float_projection);
+
+    void compute_volume(ConeProperties& ToCompute);
+    void compute_euclidean_volume(const vector<Integer>& Grad, Integer GradDenom);
+    
+    void try_multiplicity_by_descent(ConeProperties& ToCompute);
+    
+    void compute_projection(ConeProperties& ToCompute);
+    void compute_projection_from_gens(const vector<Integer>& GradOrDehom);
+    void compute_projection_from_constraints(const vector<Integer>& GradOrDehom, ConeProperties& ToCompute);
+ 
+    //in order to avoid getRank fromm inside compute
+    size_t get_rank_internal();
+    const Sublattice_Representation<Integer>& get_sublattice_internal();
 };
 
 // helpers
@@ -503,6 +606,78 @@ void insert_zero_column(vector< vector<Integer> >& mat, size_t col);
 template<typename Integer>
 void insert_column(vector< vector<Integer> >& mat, size_t col, Integer entry);
 
+// computes approximating lattice simplex using the A_n dissection of the unit cube
+// q is a rational vector with the denominator in the FIRST component q[0]
+template<typename Integer>
+void approx_simplex(const vector<Integer>& q, std::list<vector<Integer> >& approx, const long approx_level){
+	
+	//cout << "approximate the point " << q;
+    long dim=q.size();
+    long l = approx_level;
+    //if (approx_level>q[0]) l=q[0]; // approximating on level q[0](=grading) is the best we can do
+    // TODO in this case, skip the rest and just approximate on q[0]
+    Matrix<Integer> quot =  Matrix<Integer>(l,dim);
+    Matrix<Integer> remain=Matrix<Integer>(l,dim);
+    for(long j=0;j<approx_level;j++){
+	    for(long i=0;i<dim;++i){
+	        quot[j][i]=(q[i]*(j+1))/q[0];          // write q[i]=quot*q[0]+remain
+	        //quot[j][0] = 1;
+	        remain[j][i]=(q[i]*(j+1))%q[0];  // with 0 <= remain < q[0]
+	        if(remain[j][i]<0){
+	            remain[j][i]+=q[0];
+	            quot[j][i]--;
+	        }
+	          
+	    }
+	    v_make_prime(quot[j]);
+	    remain[j][0]=q[0];  // helps to avoid special treatment of i=0
+	}
+	// choose best level
+	//cout << "this is the qout matrix" << endl;
+	//quot.pretty_print(cout);
+	//cout << "this is the remain matrix" << endl;
+	//remain.pretty_print(cout);
+	long best_level=l-1;
+	vector<long> nr_zeros(l);
+	for(long j=l-1;j>=0;j--){
+		for(long i=0;i<dim;++i){
+			if(remain[j][i]==0) nr_zeros[j]++;
+		}
+		if (nr_zeros[j]>nr_zeros[best_level]) best_level=j;
+	}
+	//cout << "the best level is " << (best_level+1) << endl;
+	//now we proceed as before
+	vector<pair<Integer,size_t> > best_remain(dim);
+	for(long i=0;i<dim;i++){
+		best_remain[i].first = remain[best_level][i];
+		best_remain[i].second = i; // after sorting we must lnow where elements come from
+	}
+
+    sort(best_remain.begin(),best_remain.end()); 
+    reverse(best_remain.begin(),best_remain.end()); // we sort remain into descending order
+    
+    /*for(long i=0;i<dim;++i){
+        cout << remain[i].first << " " << remain[i].second << endl;
+    } */
+    
+    for(long i=1;i<dim;++i){
+        if(best_remain[i].first<best_remain[i-1].first)
+        {
+            approx.push_back(quot[best_level]);
+            //cout << "add the point " << quot[best_level];
+            // cout << i << " + " << remain[i].first << " + " << quot << endl;
+        }
+        quot[best_level][best_remain[i].second]++;    
+    }
+    if(best_remain[dim-1].first > 0){
+        // cout << "E " << quot << endl;
+        approx.push_back(quot[best_level]);
+        //cout << "add the point " << quot[best_level];
+    }
+
+}
+
+
 }  //end namespace libnormaliz
 
 #endif /* CONE_H_ */
